==> pawm-2.2.4.tar.bz2: pawm-2.2.4/src/pabar.c <==
/*
    Filename: pabar.c 
    Synopsys: PA auto-hide bar for labels of minimized windows
 
         	              Copyright (C) 2001-2006  
            Ral Nez de Arenas Coronado <raul@pleyades.net>
                 David Gmez Espinosa <david@pleyades.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <unistd.h>
#include <math.h>
#include <time.h>
#include "pafont.h"
#include "pabar.h"


#define LABELMARGIN 5
#define PASHUT_W  BUTTONSZ
#define PASHUT_Y ((PABARH-20)/2)
#define MINLABELS 4

#define PABAR_EVMASK (EnterWindowMask|LeaveWindowMask|ExposureMask)

/* Global data for the pabar */
static struct {
	Bool pashut_state;
	Bool pabar_scroll;
	xImage normalface;
	xImage activeface;
	xImage pressedface;
	Bool autohide;
	unsigned int clockw;
	int pashutx;
} class;

/* PABar struct */
struct sPABar {
	Window window;     /* The Puto Amo bar(tm) window */
	GC gc;
	unsigned int w;    /* Width per label */
	Chain labels;      /* List of labels for iconized pawindows */
	Button shutbutton; /* Label to shutdown the window manager */
	int y;             /* PABar y coordinate */
};

typedef struct sPABar *PABar;
#define PABarBytes (sizeof(struct sPABar))

/* Global objects */
PABar thePABar=NULL;
static unsigned short int reference_count=0;

/* NOTE: just *one* instance of this object is allowed. So, it doesn't
have a *_create nor a *_destroy member functions. The only instance it's
called 'thePABar'.  */
static void pabar_handler (XEvent *, void *);
static void pabar_EnterNotify (XEnterWindowEvent *);
static void pabar_LeaveNotify (XLeaveWindowEvent *);
static void pabar_Expose (XExposeEvent *);

static void draw_all (void);
static void map_all_labels(void);
static void unmap_all_labels(void);



void pabar_init (void) 
{
    char *tmpvalue;
    PAFont tmpfont;
	CARD32 data[12]={0, 0, 0, PABARH, 0, 0, 0, 0, 0, 0, 0, DWIDTH }; /* _NET_WM_STRUT data*/
	CARD32 data_autohide[12]={0, 0, 0, PABARBORDERH, 0, 0, 0, 0, 0, 0, 0, DWIDTH }; /* _NET_WM_STRUT data*/

    if (reference_count++) return;

    tmpvalue=cfgpool_getstring(pawmpool, "pabar_autohide");
    if (tmpvalue==NULL) die(NOTFOUNDMSG, "pabar_autohide");
    class.autohide=(strcmp(tmpvalue,"On")?False:True);
    free(tmpvalue);

    tmpfont=pafont_create(display,"palabel");
    class.clockw=0;
    pafont_destroy(tmpfont);
    
    //tmpvalue=cfgpool_getstring(pawmpool, "paclock_module");
    //if (tmpvalue==NULL) die(NOTFOUNDMSG, "paclock_module");
    //if (strcmp(tmpvalue,"On")==0) {
    //	class.pashutx=DWIDTH-class.clockw-PASHUT_W-LABELMARGIN;
    //    class.usefulwidth=(DWIDTH-class.clockw-PASHUT_W-LABELMARGIN*3);
    //} else {
	class.pashutx=DWIDTH-PASHUT_W-LABELMARGIN;
    //}
    //free(tmpvalue);

    thePABar=malloc(PABarBytes);
    if (!thePABar) die("Not enough memory");

    thePABar->gc=XCreateGC(display, ROOT, 0, NULL);
    if (thePABar->gc==None) BUG();
 
    thePABar->labels=chain_create();

    XSetForeground(display, thePABar->gc, x_color("pabar_border"));
    XSetBackground(display, thePABar->gc, x_color("pabar_base_color"));
    XSetGraphicsExposures(display, thePABar->gc, False);

    if (class.autohide)
        thePABar->window=XCreateSimpleWindow(display, ROOT, 0, LOWBAR, DWIDTH, PABARH, 0,
		     	                             BLACK, x_color("frame_base_color"));
    else
        thePABar->window=XCreateSimpleWindow(display, ROOT, 0, TOPBAR, DWIDTH, PABARH, 0,
		     	                             BLACK, x_color("frame_base_color"));

    if (thePABar->window==None) BUG();

    XStoreName(display, thePABar->window, "pabar");
	
	if (class.autohide)
		x_set_property(thePABar->window, atoms.net_wm_strut_partial, XA_CARDINAL, (unsigned char *)&data_autohide, 12);
	else 
		x_set_property(thePABar->window, atoms.net_wm_strut_partial, XA_CARDINAL, (unsigned char *)&data, 12);
	
    tmpvalue=cfgpool_getstring(pawmpool, "pashut_module");
    if (tmpvalue==NULL) die(NOTFOUNDMSG, "pashut_module");
    class.pashut_state=(strcmp(tmpvalue,"On")?False:True);
    free(tmpvalue);
    tmpvalue=cfgpool_getstring(pawmpool, "pabar_scroll");
    if (tmpvalue==NULL) die(NOTFOUNDMSG, "pabar_scroll");
    class.pabar_scroll=(strcmp(tmpvalue,"On")?False:True);
    free(tmpvalue);
    
    if (class.pashut_state) {
		char *tmpvalue;

		tmpvalue=cfgpool_getstring(pawmpool,"pashut_icon_normal");
		if (tmpvalue==NULL) die(NOTFOUNDMSG, "pashut_icon_normal");
		class.normalface=x_build_pixmap(tmpvalue);
		free(tmpvalue);
		if (class.normalface==NULL) BUG();
		
		tmpvalue=cfgpool_getstring(pawmpool,"pashut_icon_active");
		if (tmpvalue==NULL) die(NOTFOUNDMSG, "pashut_icon_active");
		class.activeface=x_build_pixmap(tmpvalue);
		free(tmpvalue);
		if (class.activeface==NULL) BUG();
		
		tmpvalue=cfgpool_getstring(pawmpool,"pashut_icon_pressed");
		if (tmpvalue==NULL) die(NOTFOUNDMSG, "pashut_icon_pressed");
		class.pressedface=x_build_pixmap(tmpvalue);
		free(tmpvalue);
		if (class.pressedface==NULL) BUG();
		
		thePABar->shutbutton=button_create(thePABar->window,class.pashutx,PASHUT_Y, BUTTONSZ,BUTTONSZ,
							   class.normalface->pixmap, class.activeface->pixmap,
							   class.pressedface->pixmap, try_to_shut,NULL);
		button_map(thePABar->shutbutton);
    }

    x_addsloot(thePABar->window, pabar_handler, NULL);
    XSelectInput(display, thePABar->window, PABAR_EVMASK);
    XMapRaised(display, thePABar->window);

}



void pabar_done (void)
{
	if (--reference_count) return;

	chain_delete(thePABar->labels);
	x_delsloot(thePABar->window, pabar_handler, NULL);    
	XUnmapWindow(display, thePABar->window);

	XFreeGC(display, thePABar->gc);
	memset(thePABar, '\0', PABarBytes);
	free(thePABar);

	thePABar=NULL;

	x_destroy_pixmap(class.normalface);
	x_destroy_pixmap(class.activeface);
	x_destroy_pixmap(class.pressedface);
}




void pabar_addlabel (PALabel palabel) 
{
	ASSERT(palabel);

	unmap_all_labels();

	/* Place the new label at the end of the chain */
	chain_seek(thePABar->labels, chain_size(thePABar->labels));
	chain_put(thePABar->labels, palabel);
	palabel_reparent(palabel, thePABar->window, 0, PABARMARGIN);

	map_all_labels();
}



void pabar_dellabel (PALabel palabel)
{
	PALabel onelabel;

	ASSERT(palabel);

	unmap_all_labels();

	chain_home(thePABar->labels);
	while ( (onelabel=chain_next(thePABar->labels))!=NULL ) {
	if (onelabel==palabel)
		chain_del(thePABar->labels);
	}

	map_all_labels();
}


void pabar_resize ()
{
	SAY("Resizing pabar for %dx%d display", DWIDTH, DHEIGHT);
	XMoveWindow(display, thePABar->window,0,DHEIGHT-1);
	XResizeWindow(display, thePABar->window, DWIDTH, PABARH);
	class.pashutx=DWIDTH-PASHUT_W-LABELMARGIN;
	button_move(thePABar->shutbutton, class.pashutx, PASHUT_Y);

	unmap_all_labels();
	map_all_labels();
}


static void pabar_handler(XEvent *xevent, void *widget) 
{
	ASSERT(xevent);

	/* There is only one pabar, so widget is not needed */
	widget=NULL;

	switch(xevent->type) {
		case EnterNotify:
			GOT(EnterNotify, PABar);
			pabar_EnterNotify(&(xevent->xcrossing));
			break;

		case LeaveNotify:
			GOT(LeaveNotify, PABar);
			pabar_LeaveNotify(&(xevent->xcrossing));
			break;

		case Expose:
			GOT(Expose, PABar);
			pabar_Expose(&(xevent->xexpose));
			break;
	}
}



static void pabar_EnterNotify (XEnterWindowEvent *event)
{
    double x=0, y=0;
    int pos=0;
    XEvent junk;
    struct timespec sleeptime;

    ASSERT(event);

    if (!class.autohide) return;

    if (event->detail==NotifyInferior) return;
    /* PABar's already up, do nothing */
    if (thePABar->y==(int)TOPBAR) return;

    thePABar->y=TOPBAR;

    /* To scroll or not to scroll... */
    if (!class.pabar_scroll) {
        XMoveWindow(display, thePABar->window, 0, (int) TOPBAR);
	return;
    }
	   
    /* Adjust x to change the speed */
    x=2; pos=LOWBAR;

    XMapRaised(display, thePABar->window);

    if (class.pashut_state)
        button_unmap(thePABar->shutbutton);

    while (pos > (int) TOPBAR) {
        y=exp(x);
        pos-=(floor(y)*0.000001);
		/* It's ok if nanosleep is interrupted, we don't sleep more time 
		 * so don't need to check remaining time */
		sleeptime.tv_sec=0; sleeptime.tv_nsec=10000000;
        nanosleep(&sleeptime,NULL);
        if (pos <= (int) TOPBAR) pos=TOPBAR;
        x++;

        XMoveWindow(display, thePABar->window, 0, pos);
        draw_all();
    }

    /* This is to avoid a race-condition when too many Enter and Leave
    events are generated before the pabar finish it drawing. We put back
    in the queue the last LeaveNotify event to hide the pabar if the
    pointer is outside . By the way: X sucks */
    while (XCheckWindowEvent(display, thePABar->window,
			     EnterWindowMask|LeaveWindowMask, &junk)==True);
        if (junk.type==LeaveNotify) XPutBackEvent(display, &junk);

    if (class.pashut_state)
        button_map(thePABar->shutbutton);
    
}



static void pabar_LeaveNotify (XLeaveWindowEvent *event)
{
	Window root, window;
	int x1, y1, x2, y2;
	unsigned int tmpmask;

	ASSERT(event);

	if (!class.autohide) return;
	/* Ignore virtual crossing between PABar and childs */
	if (event->detail==NotifyInferior) return;
	/* Check where the pointer is...*/
	if (XQueryPointer(display, ROOT, &root, &window,
					  &x1, &y1, &x2, &y2, &tmpmask)==False) BUG();
				  
	if (y1>=(int)TOPBAR) return; 		      

	XMoveWindow(display, thePABar->window, 0, LOWBAR);
	thePABar->y=LOWBAR;

	draw_all();
}


static void pabar_Expose (XExposeEvent *event) 
{
	ASSERT(event);

	if (event->count) return;

	draw_all();
}    


static void draw_all (void)
{
	XSetForeground(display, thePABar->gc, BLACK);
	XDrawLine(display, thePABar->window, thePABar->gc, 0, 0, DWIDTH, 0);
	XSetForeground(display, thePABar->gc, x_color("pabar_border"));
	XDrawLine(display, thePABar->window, thePABar->gc, 0, 1, DWIDTH, 1);
}



static void map_all_labels() 
{
	PALabel onelabel;
	int usefulwidth;
	int i=0;

	usefulwidth=(DWIDTH-PASHUT_W-LABELMARGIN*3);
	/* We don't want huge labels , so we force a minimal number of labels 
	   to calculate the width */
	if ( chain_size(thePABar->labels) < MINLABELS ) 
		thePABar->w= usefulwidth / MINLABELS;
	else
		thePABar->w= usefulwidth / chain_size(thePABar->labels);

	chain_home(thePABar->labels);
	while ( (onelabel=chain_next(thePABar->labels))!=NULL ) {
		palabel_move(onelabel,(int) thePABar->w*i+LABELMARGIN,PABARMARGIN+PABARBORDERH);
		palabel_map(onelabel,thePABar->w);
		i++;
	}

}



static void unmap_all_labels()
{
	PALabel onelabel;

	chain_home(thePABar->labels);
	while ( (onelabel=chain_next(thePABar->labels))!=NULL ) {
		palabel_unmap(onelabel);
	}
}


/* Helpers & accesors */
void pabar_map()
{
	if (class.autohide)
	XMapRaised(display, thePABar->window);
}

void pabar_unmap()
{
	XUnmapWindow(display, thePABar->window);
}

Window pabar_getwindow()
{
	return thePABar->window;
}

==> pawm-2.2.7.tar.bz2: pawm-2.2.7/src/pabar.c <==
/*
    Filename: pabar.c 
    Synopsys: PA auto-hide bar for labels of minimized windows
 
         	              Copyright (C) 2001-2006  
            Ral Nez de Arenas Coronado <raul@pleyades.net>
                 David Gmez Espinosa <david@pleyades.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdio.h>
#include <unistd.h>
#include <math.h>
#include <time.h>
#include "pafont.h"
#include "pabar.h"


#define LABELMARGIN 5
#define PASHUT_W  BUTTONSZ
#define PASHUT_Y ((PABARH-20)/2)
#define MINLABELS 4

#define PABAR_EVMASK (EnterWindowMask|LeaveWindowMask|ExposureMask)

/* Global data for the pabar */
static struct {
	Bool pashut_state;
	Bool pabar_scroll;
	xImage normalface;
	xImage activeface;
	xImage pressedface;
	Bool autohide;
	//unsigned int clockw;
	int pashutx;
} class;

/* PABar struct */
struct sPABar {
	Window window;     /* The Puto Amo bar(tm) window */
	unsigned int w;    /* Width per label */
	Chain labels;      /* List of labels for iconized pawindows */
	Button shutbutton; /* Label to shutdown the window manager */
	int y;             /* PABar y coordinate */
};

typedef struct sPABar *PABar;
#define PABarBytes (sizeof(struct sPABar))

/* Global objects */
PABar thePABar=NULL;
static unsigned short int reference_count=0;

/* NOTE: just *one* instance of this object is allowed. So, it doesn't
have a *_create nor a *_destroy member functions. The only instance it's
called 'thePABar'.  */
static void pabar_handler (XEvent *, void *);
static void pabar_EnterNotify (XEnterWindowEvent *);
static void pabar_LeaveNotify (XLeaveWindowEvent *);
static void pabar_Expose (XExposeEvent *);

static void draw_all (void);
static void map_all_labels(void);
static void unmap_all_labels(void);



void pabar_init (void) 
{
    char *tmpvalue;
    //PAFont tmpfont;
	CARD32 data[12]={0, 0, 0, PABARH, 0, 0, 0, 0, 0, 0, 0, DWIDTH }; /* _NET_WM_STRUT data*/
	CARD32 data_autohide[12]={0, 0, 0, PABARBORDERH, 0, 0, 0, 0, 0, 0, 0, DWIDTH }; /* _NET_WM_STRUT data*/

    if (reference_count++) return;

    tmpvalue=cfgpool_getstring(pawmpool, "pabar_autohide");
    if (tmpvalue==NULL) die(NOTFOUNDMSG, "pabar_autohide");
    class.autohide=(strcmp(tmpvalue,"On")?False:True);
    free(tmpvalue);

    //tmpfont=pafont_create(display, visuals.visual, "palabel");
    //class.clockw=0;
    //pafont_destroy(tmpfont);
    
    //tmpvalue=cfgpool_getstring(pawmpool, "paclock_module");
    //if (tmpvalue==NULL) die(NOTFOUNDMSG, "paclock_module");
    //if (strcmp(tmpvalue,"On")==0) {
    //	class.pashutx=DWIDTH-class.clockw-PASHUT_W-LABELMARGIN;
    //    class.usefulwidth=(DWIDTH-class.clockw-PASHUT_W-LABELMARGIN*3);
    //} else {
	class.pashutx=DWIDTH-PASHUT_W-LABELMARGIN;
    //}
    //free(tmpvalue);

    thePABar=malloc(PABarBytes);
    if (!thePABar) die("Not enough memory");

    thePABar->labels=chain_create();

    XSetForeground(display, xgc, x_color("pabar_border"));
    XSetBackground(display, xgc, x_color("pabar_base_color"));
    XSetGraphicsExposures(display, xgc, False);

    if (class.autohide)
        thePABar->window=XCreateSimpleWindow(display, ROOT, 0, LOWBAR, DWIDTH, PABARH, 0,
		     	                             BLACK, x_color("frame_base_color"));
    else
        thePABar->window=XCreateSimpleWindow(display, ROOT, 0, TOPBAR, DWIDTH, PABARH, 0,
		     	                             BLACK, x_color("frame_base_color"));

    if (thePABar->window==None) BUG();

    XStoreName(display, thePABar->window, "pabar");
	
	if (class.autohide)
		x_set_property(thePABar->window, atoms.net_wm_strut_partial, XA_CARDINAL, (unsigned char *)&data_autohide, 12);
	else 
		x_set_property(thePABar->window, atoms.net_wm_strut_partial, XA_CARDINAL, (unsigned char *)&data, 12);
	
    tmpvalue=cfgpool_getstring(pawmpool, "pashut_module");
    if (tmpvalue==NULL) die(NOTFOUNDMSG, "pashut_module");
    class.pashut_state=(strcmp(tmpvalue,"On")?False:True);
    free(tmpvalue);
    tmpvalue=cfgpool_getstring(pawmpool, "pabar_scroll");
    if (tmpvalue==NULL) die(NOTFOUNDMSG, "pabar_scroll");
    class.pabar_scroll=(strcmp(tmpvalue,"On")?False:True);
    free(tmpvalue);
    
    if (class.pashut_state) {
		char *tmpvalue;

		tmpvalue=cfgpool_getstring(pawmpool,"pashut_icon_normal");
		if (tmpvalue==NULL) die(NOTFOUNDMSG, "pashut_icon_normal");
		class.normalface=x_build_pixmap(tmpvalue);
		free(tmpvalue);
		if (class.normalface==NULL) BUG();
		
		tmpvalue=cfgpool_getstring(pawmpool,"pashut_icon_active");
		if (tmpvalue==NULL) die(NOTFOUNDMSG, "pashut_icon_active");
		class.activeface=x_build_pixmap(tmpvalue);
		free(tmpvalue);
		if (class.activeface==NULL) BUG();
		
		tmpvalue=cfgpool_getstring(pawmpool,"pashut_icon_pressed");
		if (tmpvalue==NULL) die(NOTFOUNDMSG, "pashut_icon_pressed");
		class.pressedface=x_build_pixmap(tmpvalue);
		free(tmpvalue);
		if (class.pressedface==NULL) BUG();
		
		thePABar->shutbutton=button_create(thePABar->window,class.pashutx,PASHUT_Y, BUTTONSZ,BUTTONSZ,
							   class.normalface->pixmap, class.activeface->pixmap,
							   class.pressedface->pixmap, try_to_shut,NULL);
		button_map(thePABar->shutbutton);
    }

    x_addsloot(thePABar->window, pabar_handler, NULL);
    XSelectInput(display, thePABar->window, PABAR_EVMASK);
    XMapRaised(display, thePABar->window);

}



void pabar_done (void)
{
	if (--reference_count) return;

	chain_delete(thePABar->labels);
	x_delsloot(thePABar->window, pabar_handler, NULL);    
	XUnmapWindow(display, thePABar->window);

	memset(thePABar, '\0', PABarBytes);
	free(thePABar);

	thePABar=NULL;

	x_destroy_pixmap(class.normalface);
	x_destroy_pixmap(class.activeface);
	x_destroy_pixmap(class.pressedface);
}




void pabar_addlabel (PALabel palabel) 
{
	ASSERT(palabel);

	unmap_all_labels();

	/* Place the new label at the end of the chain */
	chain_seek(thePABar->labels, chain_size(thePABar->labels));
	chain_put(thePABar->labels, palabel);
	palabel_reparent(palabel, thePABar->window, 0, PABARMARGIN);

	map_all_labels();
}



void pabar_dellabel (PALabel palabel)
{
	PALabel onelabel;

	ASSERT(palabel);

	unmap_all_labels();

	chain_home(thePABar->labels);
	while ( (onelabel=chain_next(thePABar->labels))!=NULL ) {
	if (onelabel==palabel)
		chain_del(thePABar->labels);
	}

	map_all_labels();
}


void pabar_resize ()
{
	SAY("Resizing pabar for %dx%d display", DWIDTH, DHEIGHT);
	XMoveWindow(display, thePABar->window,0,DHEIGHT-1);
	XResizeWindow(display, thePABar->window, DWIDTH, PABARH);
	class.pashutx=DWIDTH-PASHUT_W-LABELMARGIN;
	button_move(thePABar->shutbutton, class.pashutx, PASHUT_Y);

	unmap_all_labels();
	map_all_labels();
}


static void pabar_handler(XEvent *xevent, void *widget) 
{
	ASSERT(xevent);

	/* There is only one pabar, so widget is not needed */
	widget=NULL;

	switch(xevent->type) {
		case EnterNotify:
			GOT(EnterNotify, PABar);
			pabar_EnterNotify(&(xevent->xcrossing));
			break;

		case LeaveNotify:
			GOT(LeaveNotify, PABar);
			pabar_LeaveNotify(&(xevent->xcrossing));
			break;

		case Expose:
			GOT(Expose, PABar);
			pabar_Expose(&(xevent->xexpose));
			break;
	}
}



static void pabar_EnterNotify (XEnterWindowEvent *event)
{
    double x=0, y=0;
    int pos=0;
    XEvent junk;
    struct timespec sleeptime;

    ASSERT(event);

    if (!class.autohide) return;

    if (event->detail==NotifyInferior) return;
    /* PABar's already up, do nothing */
    if (thePABar->y==(int)TOPBAR) return;

    thePABar->y=TOPBAR;

    /* To scroll or not to scroll... */
    if (!class.pabar_scroll) {
        XMoveWindow(display, thePABar->window, 0, (int) TOPBAR);
	return;
    }
	   
    /* Adjust x to change the speed */
    x=2; pos=LOWBAR;

    XMapRaised(display, thePABar->window);

    if (class.pashut_state)
        button_unmap(thePABar->shutbutton);

    while (pos > (int) TOPBAR) {
        y=exp(x);
        pos-=(floor(y)*0.000001);
		/* It's ok if nanosleep is interrupted, we don't sleep more time 
		 * so don't need to check remaining time */
		sleeptime.tv_sec=0; sleeptime.tv_nsec=10000000;
        nanosleep(&sleeptime,NULL);
        if (pos <= (int) TOPBAR) pos=TOPBAR;
        x++;

        XMoveWindow(display, thePABar->window, 0, pos);
        draw_all();
    }

    /* This is to avoid a race-condition when too many Enter and Leave
    events are generated before the pabar finish it drawing. We put back
    in the queue the last LeaveNotify event to hide the pabar if the
    pointer is outside . By the way: X sucks */
    while (XCheckWindowEvent(display, thePABar->window,
			     EnterWindowMask|LeaveWindowMask, &junk)==True);
        if (junk.type==LeaveNotify) XPutBackEvent(display, &junk);

    if (class.pashut_state)
        button_map(thePABar->shutbutton);
    
}



static void pabar_LeaveNotify (XLeaveWindowEvent *event)
{
	Window root, window;
	int x1, y1, x2, y2;
	unsigned int tmpmask;

	ASSERT(event);

	if (!class.autohide) return;
	/* Ignore virtual crossing between PABar and childs */
	if (event->detail==NotifyInferior) return;
	/* Check where the pointer is...*/
	if (XQueryPointer(display, ROOT, &root, &window,
					  &x1, &y1, &x2, &y2, &tmpmask)==False) BUG();
				  
	if (y1>=(int)TOPBAR) return; 		      

	XMoveWindow(display, thePABar->window, 0, LOWBAR);
	thePABar->y=LOWBAR;

	draw_all();
}


static void pabar_Expose (XExposeEvent *event) 
{
	ASSERT(event);

	if (event->count) return;

	draw_all();
}    


static void draw_all (void)
{
	XSetForeground(display, xgc, BLACK);
	XDrawLine(display, thePABar->window, xgc, 0, 0, DWIDTH, 0);
	XSetForeground(display, xgc, x_color("pabar_border"));
	XDrawLine(display, thePABar->window, xgc, 0, 1, DWIDTH, 1);
}



static void map_all_labels() 
{
	PALabel onelabel;
	int usefulwidth;
	int i=0;

	usefulwidth=(DWIDTH-PASHUT_W-LABELMARGIN*3);
	/* We don't want huge labels , so we force a minimal number of labels 
	   to calculate the width */
	if ( chain_size(thePABar->labels) < MINLABELS ) 
		thePABar->w= usefulwidth / MINLABELS;
	else
		thePABar->w= usefulwidth / chain_size(thePABar->labels);

	chain_home(thePABar->labels);
	while ( (onelabel=chain_next(thePABar->labels))!=NULL ) {
		palabel_move(onelabel,(int) thePABar->w*i+LABELMARGIN,PABARMARGIN+PABARBORDERH);
		palabel_map(onelabel,thePABar->w);
		i++;
	}

}



static void unmap_all_labels()
{
	PALabel onelabel;

	chain_home(thePABar->labels);
	while ( (onelabel=chain_next(thePABar->labels))!=NULL ) {
		palabel_unmap(onelabel);
	}
}


/* Helpers & accesors */
void pabar_map()
{
	if (class.autohide)
	XMapRaised(display, thePABar->window);
}

void pabar_unmap()
{
	XUnmapWindow(display, thePABar->window);
}

Window pabar_getwindow()
{
	return thePABar->window;
}
